from typing import Callable

from starlette import status
from starlette.applications import Starlette
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import Response, PlainTextResponse

TOO_EARLY = PlainTextResponse("Too Early", status.HTTP_425_TOO_EARLY)


class EarlyDataMiddleware(BaseHTTPMiddleware):
    def __init__(self, app: Starlette, deny_all: bool = False):
        super().__init__(app)
        self.deny_all = deny_all

    async def dispatch(self, request: Request, call_next: Callable) -> Response:
        request.state.early_data = False
        if request.headers.get("Early-Data", None) == "1":
            if self.deny_all:
                return TOO_EARLY

            request.state.early_data = True

        response = await call_next(request)

        return response


def deny_early_data(endpoint: Callable) -> Callable:
    async def endpoint_wrapper(request: Request):
        if request.state.early_data:
            return TOO_EARLY

        return await endpoint(request)

    return endpoint_wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re

from setuptools import setup, find_packages


def get_version(package):
    """
    Return package version as listed in `__version__` in `__version__.py`.
    """
    with open(os.path.join(package, "__version__.py")) as f:
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", f.read()).group(1)


def get_long_description():
    """
    Return the README.
    """
    with open("README.md", encoding="utf8") as f:
        return f.read()


setup(
    name="starlette-early-data",
    python_requires=">=3.6",
    version=get_version("starlette_early_data"),
    url="https://github.com/HarrySky/starlette-early-data",
    license="Unlicense",
    description="Middleware and decorator for processing TLSv1.3 early data requests in Starlette",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Igor Nehoroshev",
    author_email="mail@neigor.me",
    packages=["starlette_early_data"],
    data_files=[("", ["LICENSE"])],
    include_package_data=True,
    install_requires=["starlette>=0.12.11"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: Public Domain",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
